==> fbat_1.0.0.tar.gz: fbat/R/pedHardyWeinberg.R <==
## FIXME: getAllele2 looks like it could be replaced with
##        na.omit(genotypeMat)

getAllele2 <- function(genotypeMat)
{
  nr <- nrow(genotypeMat)
  nc <- 2
  errorFlag <- 0
  ## find individuals containing missing alleles
  pos <- which(genotypeMat[, 1]*genotypeMat[, 2]==0)
  nMissing <- length(pos)
  if(nMissing>0 && nMissing<nr)
    mat <- genotypeMat[-pos, ] # individuals not contains missing alleles
  else if(nMissing==0)
    mat <- genotypeMat
  else # nMissing=nr
    stop("Alleles of all individuals are missing!\n")

  tmpvec <- as.vector(mat)
  alleles <- sortUnique(tmpvec)
  nAllele <- length(alleles)
  if(nAllele==1)
  {
    errorFlag<-1 
    stop("Only one allele!\n")
  }

  invisible(list(genotypeMat2=mat, alleles=alleles, nAllele=nAllele, 
    nMissing=nMissing, errorFlag=errorFlag, MissPos=pos)) 
}

sortUnique <- function(x)
{
  x.u <- unique(x)
  invisible(sort(x.u))
}


##FIXME: the pedHardyWeinberg() function should probably return an
##       object of class <something>, for which this should be the


## test Hardy-Weinberg equilibrium for each marker based on parental data
pedHardyWeinberg <- function(geneSetObj, threshold=3, founderOnly=TRUE, 
                             quiet=FALSE)
{
  #cat("converting geneSet object to numerical matrix...\n")
  tmp<-geneSet2Ped(geneSetObj)
  ped<-tmp$ped
  columns<-tmp$columns

  res<-pedHardyWeinberg.default (ped, columns, threshold, founderOnly, quiet)
  invisible(res)
}


## test Hardy-Weinberg equilibrium for each marker based on parental data
pedHardyWeinberg.default <- function(ped, columns=c("family", "pid", "father", "mother", "sex", "affected"), threshold=3, founderOnly=TRUE, quiet=FALSE)
{
  tmp <- checkColumn(ped, columns)
  ped<-tmp$ped
  columns<-tmp$columns
  len <- length(columns)

  ## get marker names
  markerNames <- getMarkerNames(ped, columns)

  if(founderOnly==TRUE)
  { ## get parental info
    mat1 <- ped[, c(3:4)]
    tmp <- mat1[, 1]+mat1[, 2]
    pos <- which(tmp==0)
    if(length(pos)==0)
    { stop("The pedigree data contains no parental information!\n") }
    ped <- matrix(ped[pos, ], nrow=length(pos))
  }
  mat2 <- ped

  ## for each marker, do Hardy-Weinberg equilibrium test
  nc <- ncol(mat2)
  nr <- nrow(mat2)
  nMarker <- (nc-len)/2 # number of markers
  resMat <- matrix(0, nrow=nMarker, ncol=9)
  rownames(resMat) <- markerNames
  colnames(resMat) <- c("nInfoInd", "nGenotype", "nHET", "nHOM", "nAllele", 
                        "nMissing", "chi2", "df", "p-value")
  nGenotype.vec <- rep(0, nMarker)
  for(M in 1:nMarker)
    {
      ## get the M-th marker data
      start <- len+(2*M)-1
      end <- len+(2*M)
      mat3 <- matrix(mat2[, c(start:end)], nrow=nr)
      res <- HardyWeinberg(mat3, threshold)
      ## number of informative individuals, i.e., number of genotypes which 
      ## contain no missing alleles
      resMat[M, 1]=res$nInfoInd
      nGenotype.vec[M] <- res$nGenotype # number of possible genotypes
      resMat[M, 2]=nGenotype.vec[M]  # number of different genotypes
      resMat[M, 3]=res$nHET # number of heterozygous genotype
      resMat[M, 4]=res$nHOM # number of heterozygous genotype
      resMat[M, 5]=res$nAllele # number of different alleles
      resMat[M, 6]=res$nMissing # number of genotypes contains missing allele
      resMat[M, 7]=res$chi2 # value of test statistic
      resMat[M, 8]=res$mydf # degree of freedom
      resMat[M, 9]=res$pvalue # pvalue
      alleles <- res$alleles
      errorFlag <- res$errorFlag
      if(errorFlag==2)
        {
          warning("Warning! For marker ", markerNames[M], ", some fitted values (Eij) are unreasonablly small! Hence test result might not reliable!\n");
          errorFlag<-0
        }
      if(M>1) 
        {
          if(errorFlag==0)
            {
              genotype[[M]] <- matrix(res$genotype, nrow=nGenotype.vec[M])
              colnames(genotype[[M]]) <- c(paste(markerNames[M], 1:2, sep="."), 
                                           "freq")
              piVec[[M]] <- res$piVec
              names(piVec[[M]]) <- alleles
            }
          else
            {
              genotype[[M]] <- NA
              piVec[[M]] <- NA
            }
        }
      else
        {
          if(errorFlag==0)
            {
              genotype <- list(matrix(res$genotype, nrow=nGenotype.vec[M]))
              colnames(genotype[[1]]) <- c(paste(markerNames[M], 1:2, sep="."), 
                                           "freq")
              piVec <- list(res$piVec) # allele frequency
              names(piVec[[1]]) <- alleles
            }
          else
            {
              genotype <- list(NA)
              piVec <- list(NA) # allele frequency
            }
        }
    }
  names(genotype) <- markerNames

  if(!quiet)
      print(round(resMat, 3))

  invisible(list(resMat=resMat, genotype=genotype, 
                 nGenotype.vec=nGenotype.vec,
                 markerNames=markerNames, piVec=piVec))
}


##FIXME: the pedHardyWeinberg() function should probably return an
##       object of class <something>, for which this should be the
##       show() method. 


##HW.object -- return values of the function pedHardyWeinberg()
viewHW <- function(HW.object, markerName)
{
  if(length(markerName)!=1)
    stop("The length of 'markerName' must be one!\n")

  markerNamesVec <- HW.object$markerNames
  pos <- match(markerName, markerNamesVec)
  if(is.na(pos)==TRUE)
   stop("Specified marker ", markerName, " not found\n")

  pos <- pos[1]
  resMat <- HW.object$resMat
  genotype <- HW.object$genotype
  nGenotype.vec <- HW.object$nGenotype.vec
  piVec <- HW.object$piVec

  nGenotypeM <- nGenotype.vec[pos]
  genotypeM <- genotype[[pos]]
  piVecM <- piVec[[pos]]
  resM <- resMat[pos, ]
  names(resM) <- colnames(resMat)

  cat("number of possible genotypes for marker ", markerName, " >>\n")
  print(nGenotypeM)
  cat("genotype frequency >>\n")
  print(genotypeM)
  cat("allele frequency >>\n")
  print(round(piVecM, 3))
  print(round(resM, 3))
 
  invisible(list(resM=resM, nGenotypeM=nGenotypeM, 
    genotypeM=genotypeM, piVecM=piVecM))
}


## genotypeMat -- n by 2 matrix. n row independent individuals. 2 column alleles
HardyWeinberg <- function(genotypeMat, threshold=3)
{
  ## obtain the number of alleles
  tmp <- getAllele2(genotypeMat)
  errorFlag <- tmp$errorFlag
  if(errorFlag!=0)
    return(
           list(chi2=NA, mydf=NA, pvalue=NA, alleles=NA, 
                nAllele=NA, nInfoInd=NA, nHET=NA, nHOM=NA, nMissing=NA, 
                OijMat=NA, EijMat=NA, piVec=NA, genotype=NA, 
                nGenotype=NA, errorFlag=1)
           )
  errorFlag <- 0

  genotypeMat2 <- tmp$genotypeMat2
  alleles <- tmp$alleles
  nAllele <- tmp$nAllele
  nMissing <- tmp$nMissing

  ## get Oij
  mat <- as.vector(t(genotypeMat2))
  mat <- as.integer(mat)
  nr <- as.integer(nrow(genotypeMat2))
  nc <- as.integer(2)
  alleles <- as.integer(alleles)
  nAllele <- as.integer(nAllele)
  nGenotype <- nAllele*(nAllele+1)/2 # number of possible genotypes
  nGenotype <- as.integer(nGenotype)

  ##if(!is.loaded(symbol.C("getOij"))){ dyn.load("./GeneticsBase.so") }

  res <- .C("getOij",
            mat,
            nr,
            nc,
            alleles,
            nAllele, 
            genotypeVec=as.integer(rep(0, 2*nGenotype)), 
            genotypeFreq=as.integer(rep(0, nGenotype)), 
            nGenotype=as.integer(nGenotype), 
            nHET=as.integer(0), 
            nHOM=as.integer(0), 
            OijMat=as.integer(rep(0, nAllele^2)), 
            EijMat=as.double(rep(0, nAllele^2)), 
            piVec=as.double(rep(0, nAllele)),
            chi2=as.double(0),
            PACKAGE="fbat"             
            )
  
  pos <- which(res$EijMat<threshold)
  if(length(pos)>0)
    {
      #warning("Some fitted values (Eij) are unreasonablly small!",
      #        "Hence test result might not reliable.");
      errorFlag <- 2
    }

  genotypeVec <- matrix(res$genotypeVec, nrow=nGenotype, byrow=T)
  genotypeFreq <- res$genotypeFreq
  genotype <- cbind(genotypeVec, genotypeFreq)
  colnames(genotype) <- c("1st allele", "2nd allele", "freq")

  OijMat <- matrix(res$OijMat, nrow=nAllele, byrow=T)
  EijMat <- matrix(res$EijMat, nrow=nAllele, byrow=T)
  piVec <- res$piVec
  chi2 <- res$chi2
  nHET <- res$nHET
  nHOM <- res$nHOM

  ##if(is.loaded(symbol.C("getOij"))){ dyn.unload("./GeneticsBase.so") }


  mydf <- nAllele*(nAllele-1)/2
  pvalue <- 1-pchisq(chi2, mydf) 

  invisible(list(chi2=chi2, mydf=mydf, pvalue=pvalue,
                 alleles=alleles, nAllele=nAllele, nInfoInd=nr, nHET=nHET,
                 nHOM=nHOM, nMissing=nMissing, OijMat=OijMat, EijMat=EijMat,
                 piVec=piVec, genotype=genotype, nGenotype=nGenotype,
                 errorFlag=errorFlag))

}

==> fbat_1.10.0.tar.gz: fbat/R/pedHardyWeinberg.R <==
## FIXME: getAllele2 looks like it could be replaced with
##        na.omit(genotypeMat)

getAllele2 <- function(genotypeMat)
{
  nr <- nrow(genotypeMat)
  nc <- 2
  ## find individuals containing missing alleles
  pos <- which(genotypeMat[, 1]*genotypeMat[, 2]==0)
  nMissing <- length(pos)
  if(nMissing>0 && nMissing<nr)
    mat <- genotypeMat[-pos, ,drop=FALSE] # individuals not contains missing alleles
  else if(nMissing==0)
    mat <- genotypeMat
  else # nMissing=nr
  { 
    return(list(genotypeMat2=mat, alleles=NA, nAllele=NA, 
      nMissing=nMissing, errorFlag=1, MissPos=pos)) 
  }

  tmpvec <- as.vector(mat)
  alleles <- sortUnique(tmpvec)
  nAllele <- length(alleles)
  if(nAllele==1)
  {
    return(list(genotypeMat2=mat, alleles=alleles, nAllele=nAllele, 
      nMissing=nMissing, errorFlag=2, MissPos=pos)) 
  }

  return(list(genotypeMat2=mat, alleles=alleles, nAllele=nAllele, 
    nMissing=nMissing, errorFlag=0, MissPos=pos)) 
}

sortUnique <- function(x)
{
  x.u <- unique(x)
  invisible(sort(x.u))
}


##FIXME: the pedHardyWeinberg() function should probably return an
##       object of class <something>, for which this should be the


## test Hardy-Weinberg equilibrium for each marker based on parental data
pedHardyWeinberg <- function(geneSetObj, threshold=3, founderOnly=TRUE, 
                             quiet=FALSE)
{
  if(!quiet)
    cat("converting geneSet object to numerical matrix...\n")
  pedObj<-geneSet2Ped(geneSetObj)

  if(!quiet)
    cat("HWE test...\n")

  res<-pedHardyWeinberg.default (pedObj, threshold, founderOnly, quiet)
  invisible(res)
}


## test Hardy-Weinberg equilibrium for each marker based on parental data
pedHardyWeinberg.default <- function(pedObj, threshold=3, founderOnly=TRUE, quiet=FALSE)
{
  pedObj <- checkColumn(pedObj)
  ped<-pedObj$ped
  columns<-pedObj$columns
  markerNames<-pedObj$markerNames
  nMarkers<-length(markerNames)

  len <- length(columns)

  if(founderOnly==TRUE)
  { ## get parental info
    pedObj.f<-getFounders(pedObj)
    ped<-pedObj.f$ped
  } 

  ## for each marker, do Hardy-Weinberg equilibrium test
  nc <- ncol(ped)
  nr <- nrow(ped)
  resMat <- matrix(0, nrow=nMarkers, ncol=9)
  rownames(resMat) <- markerNames
  colnames(resMat) <- c("nInfoInd", "nGenotype", "nHET", "nHOM", "nAllele", 
                        "nMissing", "chi2", "df", "p-value")
  nGenotype.vec <- rep(0, nMarkers)
  for(M in 1:nMarkers)
    {
      ## get the M-th marker data
      start <- len+(2*M)-1
      end <- len+(2*M)
      mat3 <- matrix(ped[, c(start:end)], nrow=nr)
      res <- HardyWeinberg(mat3, threshold)
      ## number of informative individuals, i.e., number of genotypes which 
      ## contain no missing alleles
      resMat[M, 1]=res$nInfoInd
      nGenotype.vec[M] <- res$nGenotype # number of possible genotypes
      resMat[M, 2]=nGenotype.vec[M]  # number of different genotypes
      resMat[M, 3]=res$nHET # number of heterozygous genotype
      resMat[M, 4]=res$nHOM # number of heterozygous genotype
      resMat[M, 5]=res$nAllele # number of different alleles
      resMat[M, 6]=res$nMissing # number of genotypes contains missing allele
      resMat[M, 7]=res$chi2 # value of test statistic
      resMat[M, 8]=res$mydf # degree of freedom
      resMat[M, 9]=res$pvalue # pvalue
      alleles <- res$alleles
      errorFlag <- res$errorFlag

      if(errorFlag==1)
        warning(paste("***** Alleles of all individuals are missing for marker",
                 markerNames[M],"!******\n",sep=""))
      else if(errorFlag==2)
        warning(paste("***** There is only one allele for marker", 
                markerNames[M],"!*****\n"))
      else
        warning(paste("For marker", markerNames[M], 
                ", some fitted values (Eij) are unreasonablly small!",
                " Hence test result might not reliable.!\n"))

      if(M>1) 
        {
          if(errorFlag==0)
            {
              genotype[[M]] <- matrix(res$genotype, nrow=nGenotype.vec[M])
              colnames(genotype[[M]]) <- c(paste(markerNames[M], 1:2, sep="."), 
                                           "freq")
              piVec[[M]] <- res$piVec
              names(piVec[[M]]) <- alleles
            }
          else
            {
              genotype[[M]] <- NA
              piVec[[M]] <- NA
            }
        }
      else
        {
          if(errorFlag==0)
            {
              genotype <- list(matrix(res$genotype, nrow=nGenotype.vec[M]))
              colnames(genotype[[1]]) <- c(paste(markerNames[M], 1:2, sep="."), 
                                           "freq")
              piVec <- list(res$piVec) # allele frequency
              names(piVec[[1]]) <- alleles
            }
          else
            {
              genotype <- list(NA)
              piVec <- list(NA) # allele frequency
            }
        }
    }
  names(genotype) <- markerNames

  if(!quiet)
      print(round(resMat, 3))

  invisible(list(resMat=resMat, genotype=genotype, 
                 nGenotype.vec=nGenotype.vec,
                 markerNames=markerNames, piVec=piVec))
}


##FIXME: the pedHardyWeinberg() function should probably return an
##       object of class <something>, for which this should be the
##       show() method. 


##HW.object -- return values of the function pedHardyWeinberg()
viewHW <- function(HW.object, markerName)
{
  if(length(markerName)!=1)
    stop("The length of 'markerName' must be one!\n")

  markerNamesVec <- HW.object$markerNames
  pos <- match(markerName, markerNamesVec)
  if(is.na(pos)==TRUE)
   stop("Specified marker ", markerName, " not found\n")

  pos <- pos[1]
  resMat <- HW.object$resMat
  genotype <- HW.object$genotype
  nGenotype.vec <- HW.object$nGenotype.vec
  piVec <- HW.object$piVec

  nGenotypeM <- nGenotype.vec[pos]
  genotypeM <- genotype[[pos]]
  piVecM <- piVec[[pos]]
  resM <- resMat[pos, ]
  names(resM) <- colnames(resMat)

  cat("number of possible genotypes for marker ", markerName, " >>\n")
  print(nGenotypeM)
  cat("genotype frequency >>\n")
  print(genotypeM)
  cat("allele frequency >>\n")
  print(round(piVecM, 3))
  print(round(resM, 3))
 
  invisible(list(resM=resM, nGenotypeM=nGenotypeM, 
    genotypeM=genotypeM, piVecM=piVecM))
}


## genotypeMat -- n by 2 matrix. n row independent individuals. 2 column alleles
HardyWeinberg <- function(genotypeMat, threshold=3)
{
  ## obtain the number of alleles
  tmp <- getAllele2(genotypeMat)
  if(tmp$errorFlag!=0)
  { 
    return(
           list(chi2=NA, mydf=NA, pvalue=NA, alleles=tmp$alleles, 
                nAllele=tmp$nAllele, nInfoInd=NA, nHET=NA, nHOM=NA, nMissing=NA, 
                OijMat=NA, EijMat=NA, piVec=NA, genotype=NA, 
                nGenotype=NA, errorFlag=tmp$errorFlag)
           )
  }
  errorFlag <- 0

  genotypeMat2 <- tmp$genotypeMat2
  alleles <- tmp$alleles
  nAllele <- tmp$nAllele
  nMissing <- tmp$nMissing

  ## get Oij
  mat <- as.vector(t(genotypeMat2))
  mat <- as.integer(mat)
  nr <- as.integer(nrow(genotypeMat2))
  nc <- as.integer(2)
  alleles <- as.integer(alleles)
  nAllele <- as.integer(nAllele)
  nGenotype <- nAllele*(nAllele+1)/2 # number of possible genotypes
  nGenotype <- as.integer(nGenotype)

  ##if(!is.loaded(symbol.C("getOij"))){ dyn.load("./GeneticsBase.so") }

  res <- .C("getOij",
            mat,
            nr,
            nc,
            alleles,
            nAllele, 
            genotypeVec=as.integer(rep(0, 2*nGenotype)), 
            genotypeFreq=as.integer(rep(0, nGenotype)), 
            nGenotype=as.integer(nGenotype), 
            nHET=as.integer(0), 
            nHOM=as.integer(0), 
            OijMat=as.integer(rep(0, nAllele^2)), 
            EijMat=as.double(rep(0, nAllele^2)), 
            piVec=as.double(rep(0, nAllele)),
            chi2=as.double(0),
            PACKAGE="fbat"             
            )
  
  pos <- which(res$EijMat<threshold)
  if(length(pos)>0)
    {
      errorFlag <- 3
    }

  genotypeVec <- matrix(res$genotypeVec, nrow=nGenotype, byrow=T)
  genotypeFreq <- res$genotypeFreq
  genotype <- cbind(genotypeVec, genotypeFreq)
  colnames(genotype) <- c("1st allele", "2nd allele", "freq")

  OijMat <- matrix(res$OijMat, nrow=nAllele, byrow=T)
  EijMat <- matrix(res$EijMat, nrow=nAllele, byrow=T)
  piVec <- res$piVec
  chi2 <- res$chi2
  nHET <- res$nHET
  nHOM <- res$nHOM

  ##if(is.loaded(symbol.C("getOij"))){ dyn.unload("./GeneticsBase.so") }


  mydf <- nAllele*(nAllele-1)/2
  pvalue <- 1-pchisq(chi2, mydf) 

  invisible(list(chi2=chi2, mydf=mydf, pvalue=pvalue,
                 alleles=alleles, nAllele=nAllele, nInfoInd=nr, nHET=nHET,
                 nHOM=nHOM, nMissing=nMissing, OijMat=OijMat, EijMat=EijMat,
                 piVec=piVec, genotype=genotype, nGenotype=nGenotype,
                 errorFlag=errorFlag))

}
